@@ -1,3 +1,6 @@
+/* global CaseDetailExpansion */
+/* global RIGHT_ARROW, DOWN_ARROW */
+
 Nitrate.TestPlans = {};
 Nitrate.TestPlans.Create = {};
 Nitrate.TestPlans.List = {};
@@ -39,386 +42,213 @@ function submitSelectedCaseIDs(url, container) {
 }
 
 Nitrate.TestPlans.TreeView = {
-  'pk': Number(),
-  'data': {},
-  'tree_elements': jQ('<div>')[0],
-  'default_container': 'id_tree_container',
+  containerId: 'treeview',
 
   /**
-   * A wrapper of jQ.ajax to filter specific plans.
+   * Find and return the element containing this tree view.
    *
-   * @param {object} data - data to send to server side.
-   * @param {Function} callback - a function called when AJAX request succeeds and the parsed
-   *                              response data will be passed in.
+   * @returns {HTMLElement} the container element.
    */
-  'filter': function (data, callback) {
-    getRequest({
-      url: '/plans/filter-for-treeview/',
-      data: Object.assign({}, data),
-      sync: true,
-      success: callback
-    });
-  },
-
-  'init': function (planId) {
-    this.pk = planId;
-
-    // Current, Parent, Brothers, Children, Temporary current
-    let curPlan, parentPlan, brotherPlans, childPlans, tempCurPlan;
-
-    // Get the current plan
-    this.filter({pk: planId}, function (responseData) {
-      if (responseData.length) {
-        curPlan = responseData[0];
-      }
-    });
-    if (!curPlan) {
-      showModal('Plan ' + planId + ' can not found in database');
-      return false;
-    }
-
-    // Get the parent plan
-    if (curPlan.parent_id) {
-      this.filter({pk: curPlan.parent_id}, function (responseData) {
-        parentPlan = responseData[0];
-      });
-    }
-
-    // Get the brother plans
-    if (curPlan.parent_id) {
-      this.filter({parent__pk: curPlan.parent_id}, function (responseData) {
-        brotherPlans = responseData;
-      });
-    }
-
-    // Get the child plans
-    this.filter({parent__pk: curPlan.pk}, function (responseData) {
-      childPlans = responseData;
-    });
-
-    // Combine all of plans
-    // Presume the plan have parent and brother at first
-    if (parentPlan && brotherPlans) {
-      parentPlan.children = brotherPlans;
-      tempCurPlan = this.traverse(parentPlan.children, curPlan.pk);
-      tempCurPlan.is_current = true;
-      if (childPlans) {
-        tempCurPlan.children = childPlans;
-      }
-
-      if (parentPlan.pk) {
-        parentPlan = Nitrate.Utils.convert('obj_to_list', parentPlan);
-      }
-
-      this.data = parentPlan;
-    } else {
-      curPlan.is_current = true;
-      if (childPlans) {
-        curPlan.children = childPlans;
-      }
-      this.data = Nitrate.Utils.convert('obj_to_list', curPlan);
-    }
+  getContainer: function () {
+    return document.getElementById(this.containerId);
   },
 
   /**
-   * An event handler will be hooked to "Up" button when render the tree.
+   * The tree view container has an element containing current plan id. This
+   * function returns the id to be used to operate the tree view.
+   *
+   * @returns {number} the plan id.
    */
-  'up': function () {
-    let tree = Nitrate.TestPlans.TreeView;
-    let parentObj = null, brotherObj = null;
+  getCurrentPlanId: function () {
+    return parseInt(
+      jQ(this.getContainer())
+        .find('input[name=plan_id]')
+        .prop('value')
+    );
+  },
 
-    tree.filter({pk: tree.data[0].parent_id}, function (responseData) {
-      parentObj = {0: responseData[0], length: 1};
-    });
-
-    tree.filter({parent__pk: tree.data[0].parent_id}, function (responseData) {
-      brotherObj = responseData;
-    });
-
-    if (parentObj && brotherObj.length) {
-      parentObj[0].children = brotherObj;
-      let brotherCount = brotherObj.length;
-      for (let i = 0; i < brotherCount; i++) {
-        if (parseInt(parentObj[0].children[i].pk) === parseInt(tree.data[0].pk)) {
-          parentObj[0].children[i] = tree.data[0];
-          break;
-        }
-      }
-      tree.data = parentObj;
-      tree.render_page();
-    }
+  getAncestors: function () {
+    return jQ(this.getContainer())
+      .find('.js-plans-treeview')
+      .jstree(true)
+      .get_node(this.getCurrentPlanId())
+      .parents
+      .filter(function (value) { return value !== '#'; })
+      .map(function (value) { return parseInt(value); });
   },
 
   /**
-   * Event handler hooked into the toggle icon click event.
+   * Get descendants of current plan
+   *
+   * @param {boolean} [direct=false] - whether to get the direct descendant plans.
+   * @returns {Array} - an array of found descendant ids.
    */
-  'blind': function () {
-    let tree = Nitrate.TestPlans.TreeView;
-    let eContainer = this;
-    let liContainer = jQ(eContainer).parent().parent();
-    let ePk = jQ(eContainer).next('a').eq(0).html();
-    let containerClns = jQ(eContainer).prop('class').split(/\s+/);
-    let expandIconUrl = '/static/images/t2.gif';
-    let collapseIconUrl = '/static/images/t1.gif';
-    let obj = tree.traverse(tree.data, ePk);
+  getDescendants: function (direct) {
+    let result = jQ(this.getContainer())
+      .find('.js-plans-treeview')
+      .jstree(true)
+      .get_node(this.getCurrentPlanId());
+    result = direct ? result.children : result.children_d;
+    return result.map(function (value) { return parseInt(value); });
+  },
 
-    containerClns.forEach(function (className) {
-      if (typeof className === 'string') {
-        switch (className) {
-          case 'expand_icon':
-            liContainer.find('ul').eq(0).hide();
-            eContainer.src = collapseIconUrl;
-            jQ(eContainer).removeClass('expand_icon').addClass('collapse_icon');
-            break;
+  /**
+   * Lazy-loading TestPlans TreeView
+   *
+   * @param {number} planId - load tree view for this plan.
+   */
+  load: function (planId) {
+    let thisView = this
+      , treeViewContainer = this.getContainer();
 
-          case 'collapse_icon':
-            if (typeof obj.children !== 'object' || obj.children === []) {
-              tree.filter({parent__pk: ePk}, function (responseData) {
-                let data = Nitrate.Utils.convert('obj_to_list', responseData);
-                tree.insert(obj, data);
-                liContainer.append(tree.render(data));
-              });
-            }
+    // An array referencing tree view nodes passed to jstree to render the plans
+    // tree view.
+    // Two purpose of setting it to null here:
+    // 1. for the first time to load the tree view, define the variable to reference the array of
+    // nodes to be rendered.
+    // 2. for subsequent tree view render, mark the old data is expired and it will reference the
+    // new value later (see below).
+    thisView.treeViewData = null;
 
-            liContainer.find('ul').eq(0).show();
-            eContainer.src = expandIconUrl;
-            jQ(eContainer).removeClass('collapse_icon').addClass('expand_icon');
-            break;
-        }
+    jQ(treeViewContainer).html('');
+    jQ(treeViewContainer).html(constructAjaxLoading());
+
+    sendHTMLRequest({
+      url: '/plan/' + planId + '/treeview/',
+      container: treeViewContainer,
+      callbackAfterFillIn: function (xhr) {
+        let planPK = parseInt(jQ('#id_tree_container').data('param'));
+
+        jQ('#js-change-parent-node').on('click', function () {
+          thisView.changeParentPlan(planPK);
+        });
+
+        jQ('#js-add-child-node').on('click', function () {
+          thisView.addChildPlan(planPK);
+        });
+
+        jQ('#js-remove-child-node').on('click', function () {
+          thisView.removeChildPlan(planPK);
+        });
+
+        jQ('div.js-plans-treeview').jstree({
+          core: {
+            themes: {
+              dots: false,
+              icons: false
+            },
+            data: thisView.treeViewData,
+          }
+        }).on('ready.jstree', function (e, data) {
+          jQ(this).jstree('open_all');
+
+          // A workaround to disable the behavior of selecting a node when click an anchor inside
+          // the node.
+          jQ(this).off('click.jstree', '.jstree-anchor');
+
+          jQ(this).on('click', 'a.js-open-cases-tab', function () {
+            Nitrate.TestPlans.Details.openTab('#testcases');
+          });
+
+          jQ(this).on('click', 'a.js-open-runs-tab', function () {
+            Nitrate.TestPlans.Details.openTab('#testruns');
+          });
+
+          let currentPlanId = thisView.getCurrentPlanId();
+
+          jQ(treeViewContainer).find('#js-remove-child-node')[0].disabled =
+            jQ(this).jstree(true).get_node(currentPlanId).children.length === 0;
+        });
       }
     });
   },
 
-  'render': function (data) {
-    let ul = jQ('<ul>');
-    let iconExpand = '<img alt="expand" src="/static/images/t2.gif" class="expand_icon js-toggle-icon">';
-    let iconCollapse = '<img alt="collapse" src="/static/images/t1.gif" class="collapse_icon js-toggle-icon">';
+  /**
+   * Add child plans to a specific test plan
+   *
+   * @param {number} currentPlanId - the parent plan of the children.
+   */
+  addChildPlan: function (currentPlanId) {
+    let inputChildPlanIds = window
+      .prompt('Enter a comma separated list of plan IDs')
+      .replace(/^[,\s]+|[,\s]+$/g, '')
+      .split(/[,\s]+/)
+      .filter(function (elem) { return elem.length > 0 });
 
-    // Add the 'Up' button
-    if (!data && this.data) {
-      data = this.data;
-      if (data && data[0].parent_id) {
-        let btn = jQ('<input>', {'type': 'button', 'value': 'Up'});
-        btn.on('click', this.up);
-        ul.append(jQ('<li>').html(btn).addClass('no-list-style'));
-      }
+    if (inputChildPlanIds.length === 0) {
+      return;
     }
 
-    // Add the child plans to parent
-    for (let i in data) {
-      let obj = data[i];
-      if (!obj.pk) { continue; }
-
-      let li = jQ('<li>');
-      let title = ['[<a href="' + obj.get_url_path + '">' + obj.pk + '</a>] '];
-
-      if (obj.num_children) {
-        li.addClass('no-list-style');
-        title.unshift(obj.children ? iconExpand : iconCollapse);
-      }
-
-      title.unshift(obj.is_active ? '<div>' : '<div class="line-through">');
-      // Construct the items
-      title.push('<a class="plan_name" href="' + obj.get_url_path + '">' + obj.name + '</a>');
-      title.push(' (');
-
-      let s = null;
-
-      if (obj.num_cases) {
-        s = obj.is_current ?
-          '<a href="#testcases" onclick="FocusTabOnPlanPage(this)">' + obj.num_cases + ' cases</a>, ' :
-          '<a href="' + obj.get_url_path + '#testcases">' + obj.num_cases + ' cases</a>, ';
-      } else {
-        s = '0 case, ';
-      }
-      title.push(s);
-
-      if (obj.num_runs) {
-        s = obj.is_current ?
-          '<a href="#testruns" onclick="FocusTabOnPlanPage(this)">' + obj.num_runs + ' runs</a>, ' :
-          '<a href="' + obj.get_url_path + '#testruns">' + obj.num_runs + ' runs</a>, ';
-      } else {
-        s = '0 runs, ';
-      }
-      title.push(s);
-
-      if (obj.num_children === 0) {
-        s = '0 child';
-      } else if (obj.num_children === 1) {
-        s = obj.is_current ?
-          '<a href="#treeview" onclick="expandCurrentPlan(jQ(this).parent()[0])">' + '1 child</a>' :
-          '<a href="' + obj.get_url_path + '#treeview">' + '1 child</a>';
-      } else {
-        let numChildren = obj.num_children;
-        s = obj.is_current ?
-          '<a href="#treeview" onclick="expandCurrentPlan(jQ(this).parent()[0])">' + numChildren + ' children</a>' :
-          '<a href="' + obj.get_url_path + '#treeview">' + numChildren + ' children</a>';
-      }
-
-      title.push(s);
-      title.push(')</div>');
-
-      ul.append(li.html(title.join('')));
-
-      if (obj.is_current) {
-        li.addClass('current');
-      }
-      if (obj.children) {
-        li.append(this.render(obj.children));
-      }
-      li.find('.js-toggle-icon').on('click', this.blind);
-    }
-
-    return ul[0];
-  },
-
-  'render_page': function (container) {
-    let _container = container || this.default_container;
-    jQ('#' + _container).html(constructAjaxLoading());
-    jQ('#' + _container).html(this.render());
-  },
-
-  'traverse': function (data, pk) {
-    // http://stackoverflow.com/questions/3645678/javascript-get-a-reference-from-json-object-with-traverse
-    for (let i in data) {
-      let obj = data[i];
-      if (obj === [] || typeof obj !== 'object') { continue; }
-      if (typeof obj.pk === 'number' && parseInt(obj.pk) === parseInt(pk)) {
-        return obj;
-      }
-
-      if (typeof obj.children === 'object') {
-        let retVal = this.traverse(obj.children, pk);
-        if (retVal !== undefined) { return retVal; }
-      }
-    }
-  },
-
-  'insert': function (node, data) {
-    if (node.children) {
-      return node;
-    }
-
-    node.children = data;
-    return node;
-  },
-
-  'toggleRemoveChildPlanButton': function () {
-    let treeContainer = jQ('#' + Nitrate.TestPlans.TreeView.default_container);
-    let tvTabContainer = Nitrate.TestPlans.Details.getTabContentContainer({
-      containerId: Nitrate.TestPlans.Details.tabContentContainerIds.treeview
-    });
-    let toEnableRemoveButton = treeContainer.find('.current').find('ul li').length > 0;
-    tvTabContainer.find('.remove_node')[0].disabled = ! toEnableRemoveButton;
-  },
-
-  'addChildPlan': function (container, currentPlanId) {
-    let self = this;
-    let tree = Nitrate.TestPlans.TreeView;
-    let childPlanIds = window.prompt('Enter a comma separated list of plan IDs').trim();
-    if (!childPlanIds) {
-      return false;
-    }
-
-    let cleanedChildPlanIds = [];
-    let inputChildPlanIds = childPlanIds.split(',');
+    let thisView = this;
+    let descendants = this.getDescendants();
 
     for (let i = 0; i < inputChildPlanIds.length; i++) {
-      let s = inputChildPlanIds[i].trim();
-      if (s === '') { continue; }
-      if (!/^\d+$/.test(s)) {
+      let item = inputChildPlanIds[i];
+      if (!/^\d+$/.test(item)) {
         showModal(
-          'Plan Id should be a numeric. ' + s + ' is not valid.', 'Add child plan'
+          'Plan id ' + item + ' is not a number. ', 'Add child plan'
         );
         return;
       }
-      let childPlanId = parseInt(s);
-      let isParentOrThisPlan = childPlanId === parseInt(tree.data[0].pk) || childPlanId === currentPlanId;
-      if (isParentOrThisPlan) {
-        showModal('Cannot add parent or self.', 'Add child plan');
+      if (descendants.indexOf(parseInt(item)) >= 0) {
+        showModal('Plan ' + item + ' has been a descendant already.');
         return;
       }
-      cleanedChildPlanIds.push(childPlanId);
     }
 
-    previewPlan({pk__in: cleanedChildPlanIds.join(',')}, '', function (e) {
+    previewPlan({pk__in: inputChildPlanIds.join(',')}, '', function (e) {
       e.stopPropagation();
       e.preventDefault();
 
-      let planId = Nitrate.Utils.formSerialize(this).plan_id;
-      updateObject({
-        contentType: 'testplans.testplan',
-        objectPk: planId,
-        field: 'parent',
-        value: currentPlanId,
-        valueType: 'int',
-        callback: function () {
-          clearDialog();
-          Nitrate.TestPlans.Details.loadPlansTreeView(currentPlanId);
-          self.toggleRemoveChildPlanButton();
-        }
-      })
-    },
-    'This operation will overwrite existing data');
-  },
+      clearDialog();  // Close the preview dialog
 
-  'removeChildPlan': function (container, currentPlanId) {
-    let self = this;
-    let tree = Nitrate.TestPlans.TreeView;
-    let childrenPks = tree.traverse(tree.data, currentPlanId).children.map(function (child) {
-      return child.pk;
-    });
-    childrenPks.sort();
-
-    let inputChildPlanIds = window.prompt('Enter a comma separated list of plan IDs to be removed');
-    if (!inputChildPlanIds) {
-      return false;
-    }
-    let cleanedChildPlanIds = [];
-    inputChildPlanIds = inputChildPlanIds.split(',');
-    for (let j = 0; j < inputChildPlanIds.length; j++) {
-      let s = inputChildPlanIds[j].trim();
-      if (s === '') { continue; }
-      if (!/^\d+$/.test(s)) {
-        showModal(
-          'Plan ID must be a number. ' + inputChildPlanIds[j] + ' is not valid.',
-          'Remove child plan'
-        )
-        return;
-      }
-      if (s === currentPlanId.toString()) {
-        showModal('Cannot remove current plan.', 'Remove child plan');
-        return;
-      }
-      if (childrenPks.indexOf(parseInt(s)) === -1) {
-        showModal('Plan ' + s + ' is not the child node of current plan', 'Remove child plan');
-        return;
-      }
-      cleanedChildPlanIds.push(s);
-    }
-
-    previewPlan({pk__in: cleanedChildPlanIds.join(',')}, '', function (e) {
-      e.stopPropagation();
-      e.preventDefault();
-
-      let planId = Nitrate.Utils.formSerialize(this).plan_id;
-      updateObject({
-        contentType: 'testplans.testplan',
-        objectPk: planId,
-        field: 'parent',
-        value: '0',
-        valueType: 'None',
-        callback: function () {
-          clearDialog();
-          Nitrate.TestPlans.Details.loadPlansTreeView(currentPlanId);
-          self.toggleRemoveChildPlanButton();
+      postRequest({
+        url: '/plan/' + currentPlanId + '/treeview/add-children/',
+        data: {'children': inputChildPlanIds},
+        traditional: true,
+        success: function () {
+          thisView.load(currentPlanId);
         }
       });
     },
     'This operation will overwrite existing data');
   },
 
-  'changeParentPlan': function (container, currentPlanId) {
+  removeChildPlan: function (currentPlanId) {
+    let thisView = this
+      , descendants = this.getDescendants();
+
+    let inputChildPlanIds = window
+      .prompt('Enter a comma separated list of plan IDs to be removed')
+      .replace(/^[,\s]+|[,\s]+$/g, '')
+      .split(/[,\s]+/)
+      .filter(function (elem) { return elem.length > 0 })
+      .filter(function (elem) { return descendants.indexOf(parseInt(elem)) >= 0 });
+
+    if (inputChildPlanIds.length === 0) {
+      return;
+    }
+
+    previewPlan({pk__in: inputChildPlanIds.join(',')}, '', function (e) {
+      e.stopPropagation();
+      e.preventDefault();
+
+      // Close the preview dialog
+      clearDialog();
+
+      postRequest({
+        url: '/plan/' + currentPlanId + '/treeview/remove-children/',
+        data: {'children': inputChildPlanIds},
+        traditional: true,
+        success: function () {
+          thisView.load(currentPlanId);
+        }
+      });
+    },
+    'This operation will overwrite existing data');
+  },
+
+  changeParentPlan: function (currentPlanId) {
     let p = prompt('Enter new parent plan ID');
     if (!p) {
       return false;
@@ -433,10 +263,19 @@ Nitrate.TestPlans.TreeView = {
       return false;
     }
 
+    let thisView = this;
+    let ancestors = thisView.getAncestors();
+    if (ancestors.indexOf(planId) >= 0) {
+      showModal('Plan ' + planId + ' has been an ancestor already.');
+      return;
+    }
+
     previewPlan({plan_id: p}, '', function (e) {
       e.stopPropagation();
       e.preventDefault();
 
+      clearDialog();
+
       let planId = Nitrate.Utils.formSerialize(this).plan_id;
       updateObject({
         contentType: 'testplans.testplan',
@@ -445,22 +284,7 @@ Nitrate.TestPlans.TreeView = {
         value: planId,
         valueType: 'int',
         callback: function () {
-          let tree = Nitrate.TestPlans.TreeView;
-          tree.filter({plan_id: p}, function (responseData) {
-            let plan = Nitrate.Utils.convert('obj_to_list', responseData);
-
-            if (tree.data[0].pk === currentPlanId) {
-              plan[0].children = jQ.extend({}, tree.data);
-              tree.data = plan;
-              tree.render_page();
-            } else {
-              plan[0].children = jQ.extend({}, tree.data[0].children);
-              tree.data = plan;
-              tree.render_page();
-            }
-
-            clearDialog();
-          });
+          thisView.load(currentPlanId);
         }
       });
     },
@@ -639,7 +463,7 @@ Nitrate.TestPlans.List.on_load = function () {
 
   jQ('#testplans_table').dataTable(
     Object.assign({}, Nitrate.TestPlans.SearchResultTableSettings, {
-      iDeferLoading: Nitrate.TestPlans.List.numberOfPlans,
+      iDeferLoading: this.numberOfPlans,
       sAjaxSource: '/plans/pages/' + this.window.location.search,
     })
   );
@@ -659,23 +483,16 @@ Nitrate.TestPlans.Details = {
     'treeview': 'treeview',
     'tag': 'tag'
   },
-  'getTabContentContainer': function (options) {
-    let constants = Nitrate.TestPlans.Details.tabContentContainerIds;
-    let id = constants[options.containerId];
-    if (id === undefined) {
-      return undefined;
-    } else {
-      return jQ('#' + id);
-    }
-  },
-  /*
-   * Lazy-loading TestPlans TreeView
+
+  /**
+   * Open a specific tab in a plan page
+   *
+   * @param {string} tabHash - the hash of a specific tab to be opened. For example, #testcases.
    */
-  'loadPlansTreeView': function (planId) {
-    // Initial the tree view
-    Nitrate.TestPlans.TreeView.init(planId);
-    Nitrate.TestPlans.TreeView.render_page();
+  openTab: function (tabHash) {
+    jQ('a[href="' + tabHash + '"]').trigger('click');
   },
+
   'initTabs': function () {
     jQ('li.tab a').on('click', function () {
       jQ('div.tab_list').hide();
@@ -698,7 +515,8 @@ Nitrate.TestPlans.Details = {
     if (!exist) {
       switchTo = defaultSwitchTo;
     }
-    jQ('a[href="' + switchTo + '"]').trigger('click');
+
+    this.openTab(switchTo);
   },
   /*
    * Load cases table.
@@ -708,8 +526,8 @@ Nitrate.TestPlans.Details = {
   'loadCases': function (container, planId, parameters) {
     constructPlanDetailsCasesZone(container, planId, parameters);
 
-    if (Nitrate.TestPlans.Details._bindEventsOnLoadedCases === undefined) {
-      Nitrate.TestPlans.Details._bindEventsOnLoadedCases = bindEventsOnLoadedCases({
+    if (this._bindEventsOnLoadedCases === undefined) {
+      this._bindEventsOnLoadedCases = bindEventsOnLoadedCases({
         'cases_container': container,
         'plan_id': planId,
         'parameters': parameters
@@ -719,7 +537,7 @@ Nitrate.TestPlans.Details = {
   // Loading newly created cases with proposal status to show table of these kind of cases.
   'loadConfirmedCases': function (planId) {
     let containerId = 'testcases';
-    Nitrate.TestPlans.Details.loadCases(containerId, planId, {
+    this.loadCases(containerId, planId, {
       'a': 'initial',
       'template_type': 'case',
       'from_plan': planId
@@ -728,39 +546,36 @@ Nitrate.TestPlans.Details = {
   // Loading reviewing cases to show table of these kind of cases.
   'loadReviewingCases': function (planId) {
     let containerId = 'reviewcases';
-    Nitrate.TestPlans.Details.loadCases(containerId, planId, {
+    this.loadCases(containerId, planId, {
       'a': 'initial',
       'template_type': 'review_case',
       'from_plan': planId
     });
   },
-  'bindEventsOnLoadedCases': function (container) {
-    let elem = typeof container === 'string' ? jQ('#' + container) : jQ(container);
-    let form = elem.children()[0];
-    let table = elem.children()[1];
-    Nitrate.TestPlans.Details._bindEventsOnLoadedCases(table, form);
-  },
+
   'observeEvents': function (planId) {
-    let NTPD = Nitrate.TestPlans.Details;
+    // let NTPD = Nitrate.TestPlans.Details;
+
+    let self = this;
 
     jQ('#tab_testcases').on('click', function () {
-      if (!NTPD.testcasesTabOpened) {
-        NTPD.loadConfirmedCases(planId);
-        NTPD.testcasesTabOpened = true;
+      if (!self.testcasesTabOpened) {
+        self.loadConfirmedCases(planId);
+        self.testcasesTabOpened = true;
       }
     });
 
     jQ('#tab_treeview').on('click', function () {
-      if (!NTPD.plansTreeViewOpened) {
-        NTPD.loadPlansTreeView(planId);
-        NTPD.plansTreeViewOpened = true;
+      if (!self.plansTreeViewOpened) {
+        Nitrate.TestPlans.TreeView.load(planId);
+        self.plansTreeViewOpened = true;
       }
     });
 
     jQ('#tab_reviewcases').on('click', function () {
-      if (!Nitrate.TestPlans.Details.reviewingCasesTabOpened) {
-        Nitrate.TestPlans.Details.loadReviewingCases(planId);
-        Nitrate.TestPlans.Details.reviewingCasesTabOpened = true;
+      if (!self.reviewingCasesTabOpened) {
+        self.loadReviewingCases(planId);
+        self.reviewingCasesTabOpened = true;
       }
     });
 
@@ -790,10 +605,10 @@ Nitrate.TestPlans.Details = {
     }
   },
   'reopenCasesTabThen': function () {
-    Nitrate.TestPlans.Details.testcasesTabOpened = false;
+    this.testcasesTabOpened = false;
   },
   'reopenReviewingCasesTabThen': function () {
-    Nitrate.TestPlans.Details.reviewingCasesTabOpened = false;
+    this.reviewingCasesTabOpened = false;
   },
   /*
    * Helper function to reopen other tabs.
@@ -803,12 +618,13 @@ Nitrate.TestPlans.Details = {
    *              Id is used to select the reopen operations.
    */
   'reopenTabHelper': function (container) {
+    let self = this;
     let switchMap = {
       'testcases': function () {
-        Nitrate.TestPlans.Details.reopenReviewingCasesTabThen();
+        self.reopenReviewingCasesTabThen();
       },
       'reviewcases': function () {
-        Nitrate.TestPlans.Details.reopenCasesTabThen();
+        self.reopenCasesTabThen();
       }
     };
     switchMap[container.prop('id')]();
@@ -849,23 +665,6 @@ Nitrate.TestPlans.Details = {
       jQ('#id_import_case_zone').hide();
       jQ('#import-error').empty();
     });
-    jQ('.js-del-attach').on('click', function () {
-      let params = jQ(this).data('params');
-      deleConfirm(params[0], 'from_plan', params[1]);
-    });
-
-    let treeview = jQ('#treeview')[0];
-    let planPK = parseInt(jQ('#id_tree_container').data('param'));
-
-    jQ('#js-change-parent-node').on('click', function () {
-      Nitrate.TestPlans.TreeView.changeParentPlan(treeview, planPK);
-    });
-    jQ('#js-add-child-node').on('click', function () {
-      Nitrate.TestPlans.TreeView.addChildPlan(treeview, planPK);
-    });
-    jQ('#js-remove-child-node').on('click', function () {
-      Nitrate.TestPlans.TreeView.removeChildPlan(treeview, planPK);
-    });
   }
 };
 
@@ -922,26 +721,28 @@ Nitrate.TestPlans.SearchCase.on_load = function () {
   });
 
   if (jQ('#id_table_cases').length) {
-    jQ('#id_table_cases').dataTable({
-      'aoColumnDefs':[{'bSortable':false, 'aTargets':[ 'nosort' ]}],
-      'aaSorting': [[ 1, 'desc' ]],
-      'sPaginationType': 'full_numbers',
-      'bFilter': false,
-      'aLengthMenu': [[10, 20, 50, -1], [10, 20, 50, 'All']],
-      'iDisplayLength': 20,
-      'bProcessing': true,
-      'fnDrawCallback': function () {
-        jQ('#id_table_cases tbody tr td:nth-child(1)').shiftcheckbox({
-          checkboxSelector: ':checkbox',
-          selectAll: '#id_table_cases .js-select-all'
-        });
+    jQ('#id_table_cases').dataTable(
+      Object.assign({}, Nitrate.DataTable.commonSettings, {
+        bServerSide: false,
 
-        jQ('#id_table_cases :checkbox').on('change', function () {
-          let disable = jQ('#id_table_cases tbody :checkbox:checked').length === 0;
-          jQ('#add-selected-cases').prop('disabled', disable);
-        });
-      }
-    });
+        aaSorting: [[1, 'desc']],
+        aoColumnDefs: [
+          {'bSortable': false, 'aTargets': ['nosort']}
+        ],
+
+        fnDrawCallback: function () {
+          jQ('#id_table_cases tbody tr td:nth-child(1)').shiftcheckbox({
+            checkboxSelector: ':checkbox',
+            selectAll: '#id_table_cases .js-select-all'
+          });
+
+          jQ('#id_table_cases :checkbox').on('change', function () {
+            let disable = jQ('#id_table_cases tbody :checkbox:checked').length === 0;
+            jQ('#add-selected-cases').prop('disabled', disable);
+          });
+        }
+      })
+    );
   }
 };
 
@@ -1001,12 +802,6 @@ Nitrate.TestPlans.Attachment.on_load = function () {
     jQ('.js-back-button').on('click', function () {
       window.history.go(-1);
     });
-
-    jQ('.js-del-attach').on('click', function () {
-      let params = jQ(this).data('params');
-      deleConfirm(params[0], params[1], params[2]);
-    });
-
   });
 };
 
@@ -1204,23 +999,15 @@ function bindEventsOnLoadedCases(options) {
     // Expand/collapse case details pane
     jQ(container).parent().find('.expandable.js-just-loaded').on('click', function () {
       let btn = this
-        , title = jQ(this).parent() // Container
-        , content = jQ(this).parent().next() // Content Containers
+        , caseDetailRow = jQ(this).parent().next() // Content Containers
         , inReviewingCasesTab = form.type.value === 'review_case';
 
-      toggleTestCasePane(
-        {
-          case_id: title.prop('id'),
-          casePaneContainer: content,
-          reviewing: inReviewingCasesTab
-        },
-        inReviewingCasesTab ? reviewCaseContentCallback(jQ(btn), content) : function () {}
+      new CaseDetailExpansion(this, inReviewingCasesTab).toggle(
+        inReviewingCasesTab ? reviewCaseContentCallback(jQ(btn), caseDetailRow) : function () {}
       );
 
-      toggleExpandArrow({caseRowContainer: title, expandPaneContainer: content});
-
-      let iconFile = areAllCasesCollapsed(inReviewingCasesTab, container) ? 't1.gif' : 't2.gif';
-      jQ(container).find('img.js-expand-collapse-cases').prop('src', '/static/images/' + iconFile);
+      let iconFile = areAllCasesCollapsed(inReviewingCasesTab, container) ? RIGHT_ARROW : DOWN_ARROW;
+      jQ(container).find('img.js-expand-collapse-cases').prop('src', iconFile);
     });
 
     /*
@@ -1327,28 +1114,6 @@ function changeCaseOrder2(parameters, callback) {
   });
 }
 
-// TODO: here
-function toggleAllCases(element) {
-  // FIXME: what does this if do?
-  //If and only if both case length is 0, remove the lock.
-  if (jQ('div[id^="id_loading_"].normal_cases').length === 0 && jQ('div[id^="id_loading_"].review_cases').length === 0){
-    jQ(element).removeClass('locked');
-  }
-
-  if (jQ(element).is('.locked')) {
-    return false;
-  } else {
-    jQ(element).addClass('locked');
-    if (jQ(element).is('.collapse-all')) {
-      element.title = 'Collapse all cases';
-      blinddownAllCases(element);
-    } else {
-      element.title = 'Expand all cases';
-      blindupAllCases(element);
-    }
-  }
-}
-
 /**
  * Load/reload the cases area which contains dropdown menus and controls.
  *
@@ -1530,19 +1295,12 @@ function constructPlanDetailsCasesZone(container, planId, parameters) {
       let allCasesExpanded = false;
 
       jQ(casesTable).find('img.js-expand-collapse-cases').on('click', function () {
-        // let iconFile = allCasesExpanded ? 't1.gif' : 't2.gif';
-        // jQ(this).prop('src', '/static/images/' + iconFile);
-
         jQ(casesTable).find('img.js-expand-collapse-case').each(function () {
           let caseTr = jQ(this).parents('tr')
             , caseDetailsTr = caseTr.next();
 
           if (caseDetailsTr.is(allCasesExpanded ? ':visible' : ':hidden')) {
             jQ(this).trigger('click');
-            toggleExpandArrow({
-              caseRowContainer: caseTr,
-              expandPaneContainer: caseDetailsTr
-            });
           }
         });
 
@@ -2107,32 +1865,6 @@ function FocusTabOnPlanPage(element) {
   jQ('#' + tabName).show();
 }
 
-/* eslint no-unused-vars: "off" */
-function expandCurrentPlan(element) {
-  let tree = Nitrate.TestPlans.TreeView;
-
-  if (jQ(element).find('.collapse_icon').length) {
-    let eContainer = jQ(element).find('.collapse_icon');
-    let liContainer = eContainer.parent().parent();
-    let ePk = eContainer.next('a').html();
-    let obj = tree.traverse(tree.data, ePk);
-
-    if (typeof obj.children !== 'object' || obj.children === []) {
-      tree.filter({parent__pk: ePk}, function (responseData) {
-        let objs = Nitrate.Utils.convert('obj_to_list', responseData);
-        tree.insert(obj, objs);
-        liContainer.append(tree.render(objs));
-      });
-    }
-
-    liContainer.find('ul').first().show();
-    eContainer
-      .prop('src', '/static/images/t2.gif')
-      .removeClass('collapse_icon')
-      .addClass('expand_icon');
-  }
-}
-
 /*
  * Handle events within Runs tab in a plan page.
  */
@@ -2170,34 +1902,35 @@ Nitrate.TestPlans.Runs = {
     if (jQ('#tab_testruns').hasClass('tab_focus')) {
       if (!jQ.fn.DataTable.fnIsDataTable(jQ('#testruns_table')[0])) {
         let url = that.makeUrlFromPlanId(jQ('#testruns_table').data('param'));
-        jQ('#testruns_table').dataTable({
-          'aoColumnDefs':[
-            {'bSortable': false, 'aTargets':[0, 8, 9, 10]},
-            {'sType': 'numeric', 'aTargets': [1, 6, 8, 9, 10 ]},
-            {'sType': 'date', 'aTargets': [5]}
-          ],
-          'bSort': true,
-          'bProcessing': true,
-          'bFilter': false,
-          'bLengthChange': false,
-          'oLanguage': {'sEmptyTable': 'No test run was found in this plan.'},
-          'bServerSide': true,
-          'sAjaxSource': url,
-          'iDisplayLength': 20,
-          'sPaginationType': 'full_numbers',
-          'fnServerParams': function (aoData) {
-            let params = jQ('#run_filter').serializeArray();
-            params.forEach(function (param) {
-              aoData.push(param);
-            });
-          },
-          'fnDrawCallback': function () {
-            jQ('#testruns_table tbody tr').shiftcheckbox({
-              checkboxSelector: ':checkbox',
-              selectAll: '#testruns_table .js-select-all'
-            });
-          }
-        });
+        jQ('#testruns_table').dataTable(
+          Object.assign({}, Nitrate.DataTable.commonSettings, {
+            sAjaxSource: url,
+
+            'aoColumnDefs':[
+              {'bSortable': false, 'aTargets':[0, 8, 9, 10]},
+              {'sType': 'numeric', 'aTargets': [1, 6, 8, 9, 10 ]},
+              {'sType': 'date', 'aTargets': [5]}
+            ],
+
+            oLanguage: {
+              sEmptyTable: 'No test run was found in this plan.'
+            },
+
+            fnServerParams: function (aoData) {
+              let params = jQ('#run_filter').serializeArray();
+              params.forEach(function (param) {
+                aoData.push(param);
+              });
+            },
+
+            fnDrawCallback: function () {
+              jQ('#testruns_table tbody tr').shiftcheckbox({
+                checkboxSelector: ':checkbox',
+                selectAll: '#testruns_table .js-select-all'
+              });
+            }
+          })
+        );
       }
     }
   },
