@@ -1,3 +1,6 @@
+/* global CaseRunDetailExpansion */
+/* global caseDetailExpansionHandler */
+
 Nitrate.TestRuns = {};
 Nitrate.TestRuns.Search = {};
 Nitrate.TestRuns.List = {};
@@ -25,41 +28,6 @@ function toggleDiv(link, divId) {
   }
 }
 
-function toggleTestCaseContents(
-  templateType, container, contentContainer, objectPk, caseTextVersion, caseRunId, callback) {
-  // TODO: should container and contentContainer be in string type?
-
-  container =
-    typeof container === 'string' ? jQ('#' + container)[0] : container;
-
-  contentContainer =
-    typeof contentContainer === 'string' ?
-      jQ('#' + contentContainer)[0] : contentContainer;
-
-  jQ(contentContainer).toggle();
-
-  if (jQ('#id_loading_' + objectPk).length) {
-    sendHTMLRequest({
-      url: Nitrate.http.URLConf.reverse({
-        name: 'case_details',
-        arguments: {id: objectPk}
-      }),
-      data: {
-        template_type: templateType,
-        case_text_version: caseTextVersion,
-        case_run_id: caseRunId
-      },
-      container: contentContainer,
-      callbackAfterFillIn: callback
-    });
-  }
-
-  toggleExpandArrow({
-    caseRowContainer: jQ(container),
-    expandPaneContainer: jQ(contentContainer)
-  });
-}
-
 /**
  * Remove a case from test run new page.
  *
@@ -106,7 +74,7 @@ Nitrate.TestRuns.Search.initializeSearchResult = function (searchEndpoint) {
     aaSorting: [[1, 'desc']],
     sAjaxSource: searchEndpoint + window.location.search,
 
-    iDeferLoading: Nitrate.TestRuns.Search.numberOfRuns,
+    iDeferLoading: this.numberOfRuns,
 
     aoColumns: [
       {'bSortable': false},       // Select checker
@@ -225,9 +193,15 @@ function showTheNumberOfCaseRunIssues(newIssuesCount, runId) {
 }
 
 
+/**
+ * Update the number of a case run's issues.
+ *
+ * @param {jQuery} caseRunRow - the container containing case run.
+ * @param {number} caseRunIssuesCount - the number of issues.
+ */
 function updateIssuesCountInCaseRunRow(caseRunRow, caseRunIssuesCount) {
   let caseRunIssuesCountSpan = jQ(caseRunRow).find('span[id$="_case_issues_count"]');
-  caseRunIssuesCountSpan.text(caseRunIssuesCount);
+  caseRunIssuesCountSpan.text(caseRunIssuesCount.toString());
   if (caseRunIssuesCount > 0) {
     caseRunIssuesCountSpan.addClass('have_issue');
   } else {
@@ -284,30 +258,30 @@ function AddIssueDialog() {
           data: data,
           traditional: true,
 
-          // After adding an issue successfully, number of issues inside the run
-          // page has to be updated and reload case run detail content eventually.
+          /**
+           * After adding an issue successfully, number of issues inside the run page has to be updated
+           * and reload case run detail content eventually.
+           *
+           * @param {object} data - server response data
+           * @param {object} data.caserun_issues_count - a mapping from case run id to the number of issues it has.
+           * @param {number} data.run_issues_count - the total number of issues of the test run.
+           */
           success: function (data) {
             // After succeeding to add issue, we close the add dialog.
             dialog.dialog('close');
 
-            let reloadInfo = dialog.dialog('option', 'reloadInfo');
+            let reloadPage = dialog.dialog('option', 'reloadPage');
 
-            // TODO: consider now to reload whole page.
-            //       consider with the else section to update partial page
-            //       content and reload expanded case run details.
-
-            if (reloadInfo.reloadPage) {
+            if (reloadPage) {
               window.location.reload();
             } else {
-              // When add issue to a case run, only need to reload the updated case run.
-              // Update issues count associated with just updated case run
-              for (let caseRunId in addIssueInfo.caseRunIds) {
-                let caseRunIssuesCount = data.caserun_issues_count[caseRunId];
-                updateIssuesCountInCaseRunRow(reloadInfo.caseRunRow, caseRunIssuesCount);
-              }
+              let expansion = dialog.dialog('option', 'caseRunDetailExpansion')
+                , caseRunId = addIssueInfo.caseRunIds[0]
+                , caseRunIssuesCount = data.caserun_issues_count[caseRunId];
+              updateIssuesCountInCaseRunRow(expansion.caseRunRow, caseRunIssuesCount);
               showTheNumberOfCaseRunIssues(data.run_issues_count, addIssueInfo.runId);
-              constructCaseRunZone(
-                reloadInfo.caseRunDetailRow, reloadInfo.caseRunRow, addIssueInfo.caseId);
+              expansion.showLoadingAnimation = true;
+              expansion.expand();
             }
           },
         });
@@ -320,7 +294,16 @@ function AddIssueDialog() {
   });
 }
 
-AddIssueDialog.prototype.open = function (addIssueInfo, reloadInfo) {
+/**
+ * Open the dialog
+ *
+ * @param {object} addIssueInfo - object containing data for adding an issue to case run(s).
+ * @param {number[]} addIssueInfo.caseRunIds - an array of case run ids.
+ * @param {number} addIssueInfo.runId - the run id.
+ * @param {CaseRunDetailExpansion} [caseRunDetailExpansion] - the detail expansion used to reload
+ *   detail content.
+ */
+AddIssueDialog.prototype.open = function (addIssueInfo, caseRunDetailExpansion) {
   if (addIssueInfo.caseRunIds === undefined || !Array.isArray(addIssueInfo.caseRunIds)) {
     throw new Error('addIssueInfo.caseRunIDs must be an array including case run IDs.');
   }
@@ -328,8 +311,10 @@ AddIssueDialog.prototype.open = function (addIssueInfo, reloadInfo) {
   let dialog = this.dialog;
 
   dialog.dialog('option', 'title', 'Add issue to case run');
-  dialog.dialog('option', 'reloadInfo', reloadInfo);
   dialog.dialog('option', 'addIssueInfo', addIssueInfo);
+  if (caseRunDetailExpansion) {
+    dialog.dialog('option', 'caseRunDetailExpansion', caseRunDetailExpansion);
+  }
 
   // Switch issue tracker tab
   dialog.find('#issue_tracker_id').change(function () {
@@ -428,9 +413,124 @@ AddEnvPropertyDialog.prototype.open = function () {
   this.dialog.dialog('open');
 }
 
-Nitrate.TestRuns.Details.on_load = function () {
-  let addIssueDialog = new AddIssueDialog();
+/**
+ * Register event handlers for the elements inside case run detail row.
+ *
+ * @param {CaseRunDetailExpansion} expansion -
+ */
+Nitrate.TestRuns.Details.registerEventHandlersForCaseRunDetail = function (expansion) {
+  let self = Nitrate.TestRuns.Details
+    , caseRunDetailRow = expansion.detailRow
+    ;
 
+  // Observe the update case run status/comment form
+  caseRunDetailRow.find('.update_form').off('submit').on('submit', function (e) {
+    e.stopPropagation();
+    e.preventDefault();
+    console.log('Update status using expansion: ', expansion);
+    updateCaseRunStatus(expansion, e.target);
+  });
+
+  caseRunDetailRow.find('.form_comment').off('submit').on('submit', function (e) {
+    e.stopPropagation();
+    e.preventDefault();
+    if (!window.confirm(defaultMessages.confirm.remove_comment)) {
+      return false;
+    }
+    removeComment(this, function () {
+      updateCommentsCount(expansion.caseId, false);
+      expansion.showLoadingAnimation = true;
+      expansion.expand();
+    });
+  });
+
+  caseRunDetailRow.find('.js-status-button').on('click', function () {
+    this.form.comment.required = false;
+    this.form.value.value = this.dataset.statusId;
+  });
+
+  caseRunDetailRow.find('.js-show-comments').on('click', function () {
+    toggleDiv(this, this.dataset.commentsElementId);
+  });
+
+  caseRunDetailRow.find('.js-show-changelog').on('click', function () {
+    toggleDiv(this, this.dataset.changelogsElementId);
+  });
+
+  caseRunDetailRow.find('.js-add-caserun-issue').on('click', function () {
+    if (self.addIssueDialog === undefined) {
+      self.addIssueDialog = new AddIssueDialog();
+    }
+    let dataset = this.dataset;
+    let addIssueInfo = {
+      runId: dataset.runId,
+      caseRunIds: [dataset.caseRunId]
+    };
+    self.addIssueDialog.open(addIssueInfo, expansion);
+  });
+
+  caseRunDetailRow.find('.js-file-issue').on('click', function () {
+    let dialogDiv = jQ('#select-tracker-dialog')
+      , form = document.forms['issueTrackerSelectionForm']
+      , fileDirectly = form.issueTrackers.options.length === 1;
+    form.action = this.dataset.action;
+    if (fileDirectly) {
+      form.submit();
+    } else {
+      // Show the dialog to select an issue tracker
+      let dialog = dialogDiv.dialog({
+        autoOpen: false,
+        resizable: false,
+        modal: true,
+        buttons: {
+          Ok: function () {
+            form.submit();
+            jQ(this).dialog('destroy');
+          },
+          Cancel: function () {
+            jQ(this).dialog('destroy');
+          }
+        }
+      });
+      dialog.dialog('open');
+    }
+  });
+
+  caseRunDetailRow.find('.js-remove-caserun-issue').on('click', function () {
+    let removeIssueInfo = jQ(this).data('params');
+    removeIssueFromCaseRuns(removeIssueInfo, function (data) {
+      let caseRunIssuesCount = data.caserun_issues_count[removeIssueInfo.caseRunIds[0]] || 0;
+      updateIssuesCountInCaseRunRow(expansion.caseRunRow, caseRunIssuesCount);
+      showTheNumberOfCaseRunIssues(data.run_issues_count, removeIssueInfo.runId);
+      expansion.showLoadingAnimation = true;
+      expansion.expand();
+    });
+  });
+
+  caseRunDetailRow.find('.js-add-testlog').on('click', function () {
+    let dialog = getAddLinkDialog();
+    dialog.dialog('option', 'target_id', expansion.caseRunId);
+    dialog.dialog('option', 'caseRunDetailExpansion', expansion);
+    dialog.dialog('open');
+  });
+
+  caseRunDetailRow.find('.js-remove-testlog').on('click', function () {
+    let self = this
+      , linkId = window.parseInt(this.dataset.linkId)
+      , url = '/linkref/remove/' + linkId + '/'
+      ;
+    postRequest({
+      url: url,
+      success: function () {
+        let li = self.parentNode
+          , ul = li.parentNode;
+        ul.removeChild(li);
+      }
+    });
+  });
+};
+
+Nitrate.TestRuns.Details.on_load = function () {
   jQ('.js-add-property').on('click', function () {
     let params = jQ(this).data('params');
     new AddEnvPropertyDialog(params[0], params[1]).open();
@@ -476,69 +576,7 @@ Nitrate.TestRuns.Details.on_load = function () {
 
   // Observe the case run toggle and the comment form
   jQ('.expandable').on('click', function () {
-    let c = jQ(this).parent(); // Case run row
-    let cContainer = c.next(); // Next row to show case run details
-    let caseId = c.find('input[name="case"]')[0].value;
-
-    /* eslint func-style:off */
-    // FIXME: move this function outside of this callback
-    let callback = function () {
-      // Observe the update case run status/comment form
-      cContainer.parent().find('.update_form').off('submit').on('submit', updateCaseRunStatus);
-
-      cContainer.parent().find('.form_comment').off('submit').on('submit', function (e) {
-        e.stopPropagation();
-        e.preventDefault();
-        if (!window.confirm(defaultMessages.confirm.remove_comment)) {
-          return false;
-        }
-        removeComment(this, function () {
-          updateCommentsCount(c.find(':hidden[name=case]').val(), false);
-          constructCaseRunZone(cContainer[0], c[0], caseId);
-        });
-      });
-
-      cContainer.find('.js-status-button').on('click', function () {
-        this.form.value.value = jQ(this).data('formvalue');
-      });
-      cContainer.find('.js-show-comments').on('click', function () {
-        toggleDiv(this, jQ(this).data('param'));
-      });
-      cContainer.find('.js-show-changelog').on('click', function () {
-        toggleDiv(this, jQ(this).data('param'));
-      });
-      cContainer.find('.js-add-caserun-issue').on('click', function () {
-        addIssueDialog.open(jQ(this).data('params'), {
-          caseRunRow: c[0],
-          caseRunDetailRow: cContainer[0]
-        });
-      });
-      cContainer.find('.js-remove-caserun-issue').on('click', function (){
-        removeIssueFromCaseRuns(jQ(this).data('params'), {
-          caseRunRow: c[0],
-          caseRunDetailRow: cContainer[0]
-        });
-      });
-      cContainer.find('.js-add-testlog').on('click', function (){
-        let params = jQ(this).data('params');
-        addLinkToCaseRun(this, params[0], params[1]);
-      });
-      cContainer.find('.js-remove-testlog').on('click', function (){
-        removeLink(this, window.parseInt(jQ(this).data('param')));
-      });
-    };
-
-    let caseRunId = c.find('input[name="case_run"]')[0].value;
-    let caseTextVersion = c.find('input[name="case_text_version"]')[0].value;
-
-    toggleTestCaseRunPane({
-      'callback': callback,
-      'caseId': caseId,
-      'caserunId': caseRunId,
-      'caseTextVersion': caseTextVersion,
-      'caserunRowContainer': c,
-      'expandPaneContainer': cContainer
-    });
+    new CaseRunDetailExpansion(this, false).toggle();
   });
 
   jQ('#id_table_cases tbody .selector_cell').shiftcheckbox({
@@ -691,19 +729,13 @@ Nitrate.TestRuns.New.on_load = function () {
     ]
   );
 
-  if (jQ('#testcases').length) {
-    jQ('#testcases').dataTable({'bPaginate': false, 'bFilter': false, 'bProcessing': true});
-  }
-
   jQ('#add_id_product_version, #add_id_build').on('click', function () {
     return popupAddAnotherWindow(this, 'product');
   });
   jQ('.js-cancel-button').on('click', function () {
     window.history.go(-1);
   });
-  jQ('.js-case-summary').on('click', function () {
-    toggleTestCaseContents(jQ(this).data('param'));
-  });
+  jQ('img.blind_icon').on('click', caseDetailExpansionHandler);
   jQ('.js-remove-case').on('click', function () {
     let params = jQ(this).data('params');
     removeItem(params[0], params[1]);
@@ -780,6 +812,8 @@ Nitrate.TestRuns.Clone.on_load = function () {
   jQ('.js-remove-button').on('click', function () {
     jQ(this).parents('.js-one-case').remove();
   });
+
+  jQ('img.blind_icon').on('click', caseDetailExpansionHandler);
 };
 
 Nitrate.TestRuns.ChooseRuns.on_load = function () {
@@ -797,18 +831,7 @@ Nitrate.TestRuns.ChooseRuns.on_load = function () {
   jQ('.js-close-help').on('click', function () {
     jQ('#help_assign').hide();
   });
-  jQ('.js-toggle-button').on('click', function () {
-    let c = jQ(this).parents('.js-one-case');
-    let cContainer = c.next();
-    let caseId = c.find('input[name="case"]').val();
-    toggleTestCasePane({'case_id': caseId, 'casePaneContainer': cContainer}, function () {
-      cContainer.children().prop('colspan', 9);
-    });
-    toggleExpandArrow({
-      'caseRowContainer': c,
-      'expandPaneContainer': cContainer
-    });
-  });
+  jQ('.js-toggle-button').on('click', caseDetailExpansionHandler);
 };
 
 Nitrate.TestRuns.AssignCase.on_load = function () {
@@ -827,96 +850,28 @@ Nitrate.TestRuns.AssignCase.on_load = function () {
   jQ('.js-close-how-assign').on('click', function () {
     jQ('#help_assign').hide();
   });
-  jQ('.js-toggle-button, .js-case-summary').on('click', function () {
-    toggleTestCaseContents(jQ(this).data('param'));
-  });
+  jQ('.js-toggle-button, .js-case-summary').on('click', caseDetailExpansionHandler);
 };
 
 /**
- * A callback called after a comment is added to a case run or a case run status is updated with a
- * short comment.
+ * A function registered to the form submit event, from where to add comment to or change status for
+ * a case run.
  *
- * @param {jQuery} caseRunRow - the container representing the case run table row.
- * @param {jQuery} expandedCaseRunDetailRow - the container representing the expanded table row
- *                                            containing case run details.
- * @param {string} caseRunStatusId - case run status ID.
+ * @param {CaseRunDetailExpansion} expansion - the case run detail expansion object.
+ * @param {HTMLElement} form - the form which triggered the event to update case run status.
  */
-function updateCaseRunDetailAfterCommentIsAdded(caseRunRow, expandedCaseRunDetailRow, caseRunStatusId) {
-  // Update the contents
-  if (caseRunStatusId !== '') {
-    // Update the case run status icon
-    let crs = Nitrate.TestRuns.CaseRunStatus;
-    caseRunRow.find('.icon_status').each(function () {
-      for (let i in crs) {
-        if (typeof crs[i] === 'string' && jQ(this).is('.btn_' + crs[i])) {
-          jQ(this).removeClass('btn_' + crs[i]);
-        }
-      }
-      jQ(this).addClass('btn_' + Nitrate.TestRuns.CaseRunStatus[parseInt(caseRunStatusId) - 1]);
-    });
-
-    // Update related people
-    caseRunRow.find('.link_tested_by').each(function () {
-      this.href = 'mailto:' + Nitrate.User.email;
-      jQ(this).html(Nitrate.User.username);
-    });
-  }
-
-  // Mark the case run to mine
-  if (!caseRunRow.is('.mine')) {
-    caseRunRow.addClass('mine');
-  }
-
-  // Blind down next case
-  let expandableElem = caseRunRow.find('.expandable');
-  expandableElem.trigger('click');
-  if (jQ('#id_check_box_auto_blinddown').prop('checked') && caseRunStatusId !== '') {
-    let nextTitle = expandedCaseRunDetailRow.next();
-    if (!nextTitle.length) {
-      return;
-    }
-    if (nextTitle.next().is(':hidden')) {
-      nextTitle.find('.expandable').trigger('click');
-    }
-  } else {
-    expandableElem.trigger('click');
-  }
-}
-
-/**
- * A function registered to the form submit event, from where to add comment to or change status for a case run.
- *
- * @param {Event} e - the DOM event.
- */
-function updateCaseRunStatus(e) {
-  e.stopPropagation();
-  e.preventDefault();
-
-  let caseRunDetailCell = jQ(this).parents().eq(3);
-  let expandedCaseRunDetailRow = caseRunDetailCell.parent();
-  let caseRunRow = expandedCaseRunDetailRow.prev();
-
-  let formData = Nitrate.Utils.formSerialize(this);
+function updateCaseRunStatus(expansion, form) {
+  let formData = Nitrate.Utils.formSerialize(form);
   let caseRunStatusId = formData.value;
 
-  // Add comment
   if (formData.comment !== '') {
-    // Reset the content to loading
-    caseRunDetailCell.html(constructAjaxLoading('id_loading_' + formData.case_id));
-
     submitComment(jQ('<div>')[0], formData, function () {
-      updateCommentsCount(caseRunRow.find(':hidden[name=case]').val(), true);
-      if (caseRunStatusId === '') {
-        updateCaseRunDetailAfterCommentIsAdded(caseRunRow, expandedCaseRunDetailRow, caseRunStatusId);
-      }
+      updateCommentsCount(expansion.caseRunRow.find(':hidden[name=case]').val(), true);
     });
   }
 
   // Update the object when changing the status
   if (caseRunStatusId !== '') {
-    // Reset the content to loading
-    caseRunDetailCell.html(constructAjaxLoading('id_loading_' + formData.case_id));
-
     updateObject({
       url: '/ajax/update/case-run-status',
       contentType: formData.content_type,
@@ -925,10 +880,56 @@ function updateCaseRunStatus(e) {
       value: caseRunStatusId,
       valueType: 'int',
       callback: function () {
-        updateCaseRunDetailAfterCommentIsAdded(caseRunRow, expandedCaseRunDetailRow, caseRunStatusId);
+        // Refresh the statistics section
+        sendHTMLRequest({
+          url: '/run/' + document.getElementById('value_run_id').value + '/statistics/',
+          container: document.getElementById('run-statistics')
+        })
+
+        // Update the case run status icon
+        let crs = Nitrate.TestRuns.CaseRunStatus;
+        expansion.caseRunRow.find('.icon_status').each(function () {
+          for (let i in crs) {
+            if (typeof crs[i] === 'string' && jQ(this).is('.btn_' + crs[i])) {
+              jQ(this).removeClass('btn_' + crs[i]);
+            }
+          }
+          jQ(this).addClass('btn_' + Nitrate.TestRuns.CaseRunStatus[parseInt(caseRunStatusId) - 1]);
+        });
+
+        // Update related people
+        expansion.caseRunRow.find('.link_tested_by').each(function () {
+          this.href = 'mailto:' + Nitrate.User.email;
+          jQ(this).html(Nitrate.User.username);
+        });
+
+        // Mark the case run to mine
+        if (! expansion.caseRunRow.is('.mine')) {
+          expansion.caseRunRow.addClass('mine');
+        }
       }
     });
   }
+
+  let expandNext =
+      jQ('#id_check_box_auto_blinddown').prop('checked') &&
+      caseRunStatusId !== '' &&
+      !expansion.atLastRow;
+
+  // TODO: the statistics has to be updated as well without reloading whole page.
+
+  if (expandNext) {
+    expansion.collapseCaseRunDetail();
+    window.setTimeout(function () {
+      expansion.expand();
+      expansion.expandNextCaseRunDetail();
+    }, 700);
+  } else {
+    expansion.showCaseRunDetailAjaxLoading();
+    window.setTimeout(function () {
+      expansion.expand();
+    }, 700);
+  }
 }
 
 function changeCaseRunOrder(runId, caseRunId, sortKey) {
@@ -999,45 +1000,30 @@ function taggleSortCaseRun(event) {
   }
 }
 
-function constructCaseRunZone(container, titleContainer, caseId) {
-  if (container) {
-    let td = jQ('<td>', {'id': 'id_loading_' + caseId, 'colspan': 12});
-    td.html(constructAjaxLoading());
-    jQ(container).html(td);
-  }
 
-  if (titleContainer) {
-    let link = jQ(titleContainer).find('.expandable');
-    link.trigger('click');
-    link.trigger('click');
-  }
-}
-
-
-function removeIssueFromCaseRuns(removeIssueInfo, reloadInfo) {
+/**
+ * Remove issue from selected case runs
+ *
+ * @param {object} removeIssueInfo - object containing data for removing an issue from case run(s).
+ * @param {number[]} removeIssueInfo.caseRunIds - an array of case runs from which to remove the issue.
+ * @param {string} removeIssueInfo.issueKey - the issue to remove.
+ * @param {number} removeIssueInfo.runId - the run id.
+ * @param {Function} successCallback - the callback registered to success.
+ */
+function removeIssueFromCaseRuns(removeIssueInfo, successCallback) {
   if (removeIssueInfo.issueKey === undefined || removeIssueInfo.issueKey === '') {
     throw new Error('Missing issue key to remove.');
   }
 
   getRequest({
-    url: '/run/' + removeIssueInfo.runId + '/issues/',
+    url: '/run/' + removeIssueInfo.runId.toString() + '/issues/',
     data: {
       a: 'remove',
       case_run: removeIssueInfo.caseRunIds,
       issue_key: removeIssueInfo.issueKey
     },
     traditional: true,
-    success: function (data) {
-      if (reloadInfo.reloadPage) {
-        window.location.reload();
-      } else {
-        let caseRunIssuesCount = data.caserun_issues_count[removeIssueInfo.caseRunId];
-        updateIssuesCountInCaseRunRow(reloadInfo.caseRunRow, caseRunIssuesCount);
-        showTheNumberOfCaseRunIssues(data.run_issues_count, removeIssueInfo.runId);
-        constructCaseRunZone(
-          reloadInfo.caseRunDetailRow, reloadInfo.caseRunRow, removeIssueInfo.caseId);
-      }
-    },
+    success: successCallback,
   });
 }
 
@@ -1290,9 +1276,9 @@ function getSelectedCaseRunIDs() {
   return result;
 }
 
-function showCaseRunsWithSelectedStatus(form, statusId) {
-  form.case_run_status__pk.value = statusId;
-  jQ(form).find('input[type="submit"]').trigger('click');
+function showCaseRunsWithSelectedStatus(form, statusName) {
+  form.case_run_status__name.value = statusName;
+  form.submit();
 }
 
 //Added for choose runs and add cases to those runs
@@ -1337,9 +1323,9 @@ function addIssueToBatchCaseRunsHandler() {
   } else {
     let addIssueInfo = jQ(this).data('addIssueInfo');
     addIssueInfo.caseRunIds = caseRunIds;
-    let reloadInfo = jQ(this).data('reloadInfo');
     let dialog = new AddIssueDialog();
-    dialog.open(addIssueInfo, reloadInfo);
+    dialog.dialog.dialog('option', 'reloadPage', true);
+    dialog.open(addIssueInfo);
   }
 }
 
@@ -1355,7 +1341,6 @@ function removeIssueFromBatchCaseRunsHandler() {
   if (caseRunIds.length === 0) {
     showModal(defaultMessages.alert.no_case_selected, 'Missing something?');
   } else {
-    let reloadInfo = jQ(this).data('reloadInfo');
     let removeIssueInfo = jQ(this).data('removeIssueInfo');
     removeIssueInfo.caseRunIds = caseRunIds;
 
@@ -1368,7 +1353,7 @@ function removeIssueFromBatchCaseRunsHandler() {
           // Don't care about closing or destroying current dialog.
           // Whole page will be reloaded.
           removeIssueInfo.issueKey = jQ(this).find('input[id=issueKeyToRemove]').val();
-          removeIssueFromCaseRuns(removeIssueInfo, reloadInfo);
+          removeIssueFromCaseRuns(removeIssueInfo);
         },
         Cancel: function () {
           jQ(this).dialog('close');
@@ -1462,41 +1447,6 @@ function getAddLinkDialog() {
   return jQ('#addlink_dialog');
 }
 
-/**
- * Do AJAX request to backend to remove a link
- *
- * @param {HTMLElement} sender - the HTML element triggering the event to remove a link.
- * @param {number} linkId - the ID of an arbitrary link.
- */
-function removeLink(sender, linkId) {
-  let url = '/linkref/remove/' + linkId + '/';
-  postRequest({url: url, success: function () {
-    let liNode = sender.parentNode;
-    liNode.parentNode.removeChild(liNode);
-  }});
-}
-
-/**
- * Add link to case run
- *
- * @param {HTMLElement} sender - the Add link button, which is pressed to fire this event.
- * @param {number} caseId - the test case ID.
- * @param {number} caseRunId - the test case run ID.
- */
-function addLinkToCaseRun(sender, caseId, caseRunId) {
-  let dialog = getAddLinkDialog();
-
-  dialog.dialog('option', 'target_id', caseRunId);
-  // These two options are used for reloading TestCaseRun when successfully.
-  let container = jQ(sender).parents('.case_content.hide')[0];
-  dialog.dialog('option', 'container', container);
-  let titleContainer = jQ(container).prev()[0];
-  dialog.dialog('option', 'title_container', titleContainer);
-  dialog.dialog('option', 'case_id', caseId);
-  dialog.dialog('open');
-}
-
-/* eslint no-unused-vars: off */
 /**
  * Initialize dialog for getting information about new link, which is attached
  * to an arbitrary instance of TestCaseRun
@@ -1532,13 +1482,9 @@ function initializeAddLinkDialog(linkTarget) {
           },
           success: function () {
             dialog.dialog('close');
-
-            // Begin to construct case run area
-            constructCaseRunZone(
-              dialog.dialog('option', 'container'),
-              dialog.dialog('option', 'title_container'),
-              dialog.dialog('option', 'case_id')
-            );
+            let expansion = dialog.dialog('option', 'caseRunDetailExpansion');
+            expansion.showLoadingAnimation = true;
+            expansion.expand();
           },
         });
       },
@@ -1562,38 +1508,3 @@ function initializeAddLinkDialog(linkTarget) {
     target_id: null
   });
 }
-
-
-/**
- * Toggle TestCaseRun panel to edit a case run in run page.
- *
- * @param {object} options - options for toggling the test case run pane.
- * @param {HTMLElement} options.caserunRowContainer - the container element containing the test case run row.
- * @param {HTMLElement} options.expandPaneContainer - the container element to be expanded for the associated test case
- *                                                    run.
- * @param {number} options.caseId - the associated test case ID.
- * @param {number} options.caserunId - the test case run ID.
- * @param {number} options.caseTextVersion - the test case' text version.
- * @param {Function} options.callback - a function called after the content is filled in the expanded container.
- */
-function toggleTestCaseRunPane(options) {
-  let container = options.expandPaneContainer;
-  container.toggle();
-
-  if (container.find('.ajax_loading').length) {
-    sendHTMLRequest({
-      url: '/case/' + options.caseId + '/caserun-detail-pane/',
-      container: container,
-      callbackAfterFillIn: options.callback,
-      data: {
-        case_run_id: options.caserunId,
-        case_text_version: options.caseTextVersion
-      },
-    });
-  }
-
-  toggleExpandArrow({
-    caseRowContainer: options.caserunRowContainer,
-    expandPaneContainer: container
-  });
-}
