@@ -413,14 +413,27 @@ class TestTag(TCMSActionModel):
 # Test attachements file zone
 
 
+def attachment_stored_filename(stored_name: str) -> str:
+    return (os.path
+            .join(settings.FILE_UPLOAD_DIR, stored_name)
+            .replace('\\', '/'))
+
+
 class TestAttachment(models.Model):
     attachment_id = models.AutoField(primary_key=True)
     description = models.CharField(max_length=1024, blank=True, null=True)
     file_name = models.CharField(db_column='filename', max_length=255, unique=True, blank=True)
     stored_name = models.CharField(max_length=128, unique=True, blank=True, null=True)
-    create_date = models.DateTimeField(db_column='creation_ts')
+    create_date = models.DateTimeField(
+        db_column='creation_ts',
+        auto_now_add=True
+    )
     mime_type = models.CharField(max_length=100)
-    # checksum = models.CharField(max_length=128, unique=True)
+    checksum = models.CharField(
+        max_length=32,
+        unique=True,
+        help_text='MD5 checksum of this uploaded file.'
+    )
 
     submitter = models.ForeignKey('auth.User',
                                   blank=True, null=True,
@@ -434,16 +447,8 @@ class TestAttachment(models.Model):
         db_table = 'test_attachments'
 
     @property
-    def stored_filename(self):
-        return (
-            os.path
-            .join(settings.FILE_UPLOAD_DIR, self.stored_name)
-            .replace('\\', '/')
-        )
-
-    @property
-    def exists(self):
-        return os.path.exists(self.stored_filename)
+    def stored_filename(self) -> str:
+        return attachment_stored_filename(self.stored_name)
 
 
 class TestAttachmentData(models.Model):
