@@ -126,8 +126,10 @@ class TestCase(TCMSActionModel):
         'auth.User', blank=True, null=True,
         related_name='cases_as_reviewer', on_delete=models.SET_NULL)
 
-    attachment = models.ManyToManyField('management.TestAttachment', related_name='cases',
-                                        through='testcases.TestCaseAttachment')
+    attachments = models.ManyToManyField(
+        'management.TestAttachment',
+        related_name='cases',
+        through='testcases.TestCaseAttachment')
 
     # FIXME: related_name should be cases instead of case. But now keep it
     # named case due to historical reason.
@@ -466,9 +468,7 @@ class TestCase(TCMSActionModel):
         return format_timedelta(self.estimated_time)
 
     def get_issues(self):
-        return (Issue.objects.filter(case__pk=self.pk)
-                             .select_related('tracker')
-                             .order_by('pk'))
+        return Issue.objects.filter(case__pk=self.pk).select_related('tracker', 'case_run')
 
     def get_choiced(self, obj_value, choices):
         for x in choices:
@@ -674,7 +674,7 @@ class TestCase(TCMSActionModel):
         if copy_attachment:
             TestCaseAttachment.objects.bulk_create([
                 TestCaseAttachment(case=cloned_case, attachment=item)
-                for item in self.attachment.all()
+                for item in self.attachments.all()
             ])
 
         rel = TestCasePlan.objects.filter(plan=source_plan, case=self).first()
