@@ -575,7 +575,7 @@ class SimpleTestCaseView(TemplateView, data.TestCaseViewDataMixin):
         if case is not None:
             data.update({
                 'test_case_text': case.get_text_with_version(),
-                'attachments': case.attachment.only('file_name'),
+                'attachments': case.attachments.only('file_name'),
                 'components': case.component.only('name'),
                 'tags': case.tag.only('name'),
                 'case_comments': self.get_case_comments(case),
@@ -757,7 +757,12 @@ class TestCaseCaseRunDetailPanelView(TemplateView,
 
         caserun_status = TestCaseRunStatus.objects.values('pk', 'name')
         caserun_status = caserun_status.order_by('sortkey')
-        issues = group_case_issues(case_run.case.get_issues().order_by('issue_key'))
+        issues = group_case_issues(case_run.case
+                                   .get_issues()
+                                   .order_by('issue_key')
+                                   .only('issue_key', 'tracker__name',
+                                         'tracker__issue_url_fmt', 'case_run__run'))
+        has_issue_trackers = case_run.run.get_issue_trackers().exists()
 
         data.update({
             'test_case': case,
@@ -769,6 +774,7 @@ class TestCaseCaseRunDetailPanelView(TemplateView,
             'caserun_logs': caserun_logs,
             'test_case_run_status': caserun_status,
             'grouped_case_issues': issues,
+            'has_issue_trackers': has_issue_trackers,
         })
 
         return data
@@ -1494,11 +1500,16 @@ class AddIssueToCases(CasesIssueActionBaseView):
     permission_required = 'issuetracker.add_issue'
 
     def do(self, form):
+        tracker = form.cleaned_data['tracker']
+        if not tracker.enabled:
+            return JsonResponseBadRequest({
+                'message': f'Issue tracker "{tracker.name}" is not enabled.'
+            })
         case = form.cleaned_data['case']
         try:
             case.add_issue(
                 issue_key=form.cleaned_data['issue_key'],
-                issue_tracker=form.cleaned_data['tracker'],
+                issue_tracker=tracker,
                 summary=form.cleaned_data['summary'],
                 description=form.cleaned_data['description'],
             )
