@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 
 from datetime import datetime
+from textwrap import dedent
+from typing import List
 
 from django.conf import settings
 from django.core.exceptions import ObjectDoesNotExist
@@ -13,6 +15,7 @@ from django.urls import reverse
 from uuslug import slugify
 
 from tcms.core.models import TCMSActionModel
+from tcms.core.raw_sql import RawSQL
 from tcms.core.utils import checksum
 from tcms.core.tcms_router import connection
 from tcms.management.models import Version
@@ -62,8 +65,10 @@ class TestPlan(TCMSActionModel):
                                related_name='child_set',
                                on_delete=models.SET_NULL)
 
-    attachment = models.ManyToManyField('management.TestAttachment',
-                                        through='testplans.TestPlanAttachment')
+    attachments = models.ManyToManyField(
+        'management.TestAttachment',
+        through='testplans.TestPlanAttachment')
+
     component = models.ManyToManyField('management.Component',
                                        through='testplans.TestPlanComponent')
     env_group = models.ManyToManyField('management.TCMSEnvGroup',
@@ -110,6 +115,22 @@ class TestPlan(TCMSActionModel):
 
         return q.filter(**new_query).distinct()
 
+    @classmethod
+    def apply_subtotal(cls, queryset,
+                       cases_count: bool = False,
+                       runs_count: bool = False,
+                       children_count: bool = False):
+        select = {}
+        if cases_count:
+            select['cases_count'] = RawSQL.num_cases
+        if runs_count:
+            select['runs_count'] = RawSQL.num_runs
+        if children_count:
+            select['children_count'] = RawSQL.num_child_plans
+        if select:
+            queryset = queryset.extra(select=select)
+        return queryset
+
     def confirmed_case(self):
         return self.case.filter(case_status__name='CONFIRMED')
 
@@ -327,7 +348,7 @@ class TestPlan(TCMSActionModel):
 
         # Copy the plan attachments
         if copy_attachments:
-            for tp_attach_src in self.attachment.all():
+            for tp_attach_src in self.attachments.all():
                 tp_dest.add_attachment(attachment=tp_attach_src)
 
         # Copy the environment group
@@ -449,6 +470,57 @@ class TestPlan(TCMSActionModel):
 
         self.add_case(tc, sortkey=sortkey)
 
+    def get_descendant_ids(self, direct: bool = False) -> List[int]:
+        if direct:
+            sql = dedent(f'''
+                WITH RECURSIVE sub_tree AS (
+                    SELECT plan_id, 0 as depth FROM test_plans WHERE plan_id = {self.pk}
+                    UNION ALL
+                    SELECT tp.plan_id, depth + 1 FROM test_plans AS tp, sub_tree AS st
+                    WHERE tp.parent_id = st.plan_id
+                )
+                SELECT plan_id FROM sub_tree WHERE depth = 1;
+            ''')
+        else:
+            sql = dedent(f'''
+                WITH RECURSIVE sub_tree AS (
+                    SELECT plan_id, 0 as depth FROM test_plans WHERE plan_id = {self.pk}
+                    UNION ALL
+                    SELECT tp.plan_id, depth + 1 FROM test_plans AS tp, sub_tree AS st
+                    WHERE tp.parent_id = st.plan_id
+                )
+                SELECT plan_id FROM sub_tree WHERE depth > 0;
+            ''')
+        with connection.reader_cursor as cursor:
+            cursor.execute(sql)
+            return [row[0] for row in cursor.fetchall()]
+
+    def get_descendants(self):
+        descendant_ids = self.get_descendant_ids()
+        return TestPlan.objects.filter(pk__in=descendant_ids)
+
+    def get_ancestor_ids(self) -> List[int]:
+        sql_ancestors = dedent(f'''
+            WITH RECURSIVE sub_tree AS (
+                SELECT plan_id, parent_id FROM test_plans
+                WHERE plan_id = {self.pk}
+                UNION ALL
+                SELECT tp.plan_id, tp.parent_id
+                FROM test_plans AS tp, sub_tree AS st
+                WHERE st.parent_id = tp.plan_id
+            )
+            SELECT * FROM sub_tree;
+        ''')
+        with connection.reader_cursor as cursor:
+            cursor.execute(sql_ancestors)
+            result = [row[0] for row in cursor.fetchall()]
+            result.remove(self.pk)
+            return result
+
+    def get_ancestors(self):
+        ancestor_ids = self.get_ancestor_ids()
+        return TestPlan.objects.filter(pk__in=ancestor_ids)
+
 
 class TestPlanText(TCMSActionModel):
     plan = models.ForeignKey(TestPlan, related_name='text', on_delete=models.CASCADE)
