@@ -30,4 +30,16 @@ urlpatterns = [
 
     path('<int:plan_id>/runs/', testruns_views.load_runs_of_one_plan,
          name='load_runs_of_one_plan_url'),
+
+    path('<int:plan_id>/treeview/',
+         views.construct_plans_treeview,
+         name='plan-treeview'),
+
+    path('<int:plan_id>/treeview/add-children/',
+         views.treeview_add_child_plans,
+         name='plan-treeview-add-children'),
+
+    path('<int:plan_id>/treeview/remove-children/',
+         views.treeview_remove_child_plans,
+         name='plan-treeview-remove-children'),
 ]
