@@ -49,13 +49,11 @@ from tcms.core.utils import DataTableResult
 from tcms.core.utils import format_timedelta
 from tcms.core.views import prompt
 from tcms.management.models import Priority, TestTag, TCMSEnvGroup
-from tcms.search.forms import RunForm
-from tcms.search.query import SmartDjangoQuery
 from tcms.testcases.models import TestCase
 from tcms.testcases.models import TestCasePlan, TestCaseStatus
 from tcms.testcases.views import get_selected_testcases
 from tcms.testplans.models import TestPlan
-from tcms.testruns.data import stats_caseruns_status
+from tcms.testruns.data import stats_case_runs_status
 from tcms.testruns.data import TestCaseRunDataMixin
 from tcms.testruns.forms import MulitpleRunsCloneForm, PlanFilterRunForm, RunAndEnvValueForm, ChangeRunEnvValueForm
 from tcms.testruns.forms import NewRunForm, SearchRunForm, EditRunForm, RunCloneForm
@@ -585,6 +583,21 @@ def open_run_get_users(case_runs):
     return (dict(testers.iterator()), dict(assignees.iterator()))
 
 
+class RunStatisticsView(TemplateView):
+    """A simple view for refreshing the statistics by case run status in a test run page"""
+
+    template_name = 'run/status_statistics.html'
+
+    def get_context_data(self, **kwargs):
+        data = super().get_context_data()
+        run_id = self.kwargs['run_id']
+        data.update({
+            'test_run': get_object_or_404(TestRun.objects.only('pk'), pk=run_id),
+            'status_stats': stats_case_runs_status([run_id])[run_id]
+        })
+        return data
+
+
 @require_GET
 def get(request, run_id, template_name='run/get.html'):
     """Display testrun's detail"""
@@ -592,22 +605,19 @@ def get(request, run_id, template_name='run/get.html'):
     SUB_MODULE_NAME = "runs"
 
     # Get the test run
-    try:
-        # 1. get test run itself
-        tr = TestRun.objects.select_related().get(run_id=run_id)
-    except ObjectDoesNotExist:
-        raise Http404
+    tr: TestRun = get_object_or_404(TestRun.objects.select_related(), pk=run_id)
 
     # Get the test case runs belong to the run
     # 2. get test run's all case runs
     tcrs = open_run_get_case_runs(request, tr)
 
-    case_run_statuss = TestCaseRunStatus.objects.only('pk', 'name')
-    case_run_statuss = case_run_statuss.order_by('pk')
+    case_run_statuss = (TestCaseRunStatus.objects
+                        .only('pk', 'name')
+                        .order_by('pk'))
 
     # Count the status
     # 3. calculate number of case runs of each status
-    status_stats_result = stats_caseruns_status(run_id, case_run_statuss)
+    status_stats_result = stats_case_runs_status([run_id])[run_id]
 
     # Get the test case run bugs summary
     # 6. get the number of bugs of this run
@@ -653,7 +663,7 @@ def get(request, run_id, template_name='run/get.html'):
         'test_case_run_status': case_run_statuss,
         'priorities': Priority.objects.all(),
         'case_own_tags': ttags,
-        'issue_trackers': tr.plan.product.issue_trackers.all(),
+        'issue_trackers': tr.get_issue_trackers(),
     }
     return render(request, template_name, context=context_data)
 
@@ -1335,59 +1345,12 @@ class ChangeRunEnvValueView(PermissionRequiredMixin, FormView):
         })
 
 
-def caseruns(request, templ='report/caseruns.html'):
-    """View that search caseruns."""
-    queries = request.GET
-    r_form = RunForm(queries)
-    r_form.populate(queries)
-    context = {}
-    if r_form.is_valid():
-        runs = SmartDjangoQuery(r_form.cleaned_data, TestRun.__name__)
-        runs = runs.evaluate()
-        caseruns = get_caseruns_of_runs(runs, queries)
-        context['test_case_runs'] = caseruns
-        context['runs'] = runs
-    return render(request, templ, context=context)
-
-
-def get_caseruns_of_runs(runs, kwargs=None):
-    """
-    Filtering argument -
-        priority
-        tester
-        plan tag
-    """
-
-    if kwargs is None:
-        kwargs = {}
-    plan_tag = kwargs.get('plan_tag', None)
-    if plan_tag:
-        runs = runs.filter(plan__tag__name=plan_tag)
-    caseruns = TestCaseRun.objects.filter(run__in=runs)
-    priority = kwargs.get('priority', None)
-    if priority:
-        caseruns = caseruns.filter(case__priority__pk=priority)
-    tester = kwargs.get('tester', None)
-    if not tester:
-        caseruns = caseruns.filter(tested_by=None)
-    if tester:
-        caseruns = caseruns.filter(tested_by__pk=tester)
-    status = kwargs.get('status', None)
-    if status:
-        caseruns = caseruns.filter(case_run_status__name__iexact=status)
-    return caseruns
-
-
 class FileIssueForCaseRun(RedirectView):
     """Construct an issue report URL and redirect to it"""
 
     def get_redirect_url(self, *args, **kwargs):
-        _, case_run_id = self.args
+        case_run_id = self.kwargs['case_run_id']
+        tracker_id = int(self.request.GET['issueTrackers'])
         case_run = get_object_or_404(TestCaseRun, pk=case_run_id)
-        # This name should be get from rendered webpage dynamically.
-        # It is hardcoded as a temporary solution right now.
-        # FIXME: name here should be RHBugzilla in final solution.
-        bz_model = IssueTracker.objects.get(name='Bugzilla')
-        # An eventual solution would be to just call this method
-        # and pass loaded issue tracker name.
+        bz_model = IssueTracker.objects.get(pk=tracker_id)
         return find_service(bz_model).make_issue_report_url(case_run)
