@@ -11,7 +11,6 @@ from unittest.mock import patch
 from bs4 import BeautifulSoup
 
 from django import test
-from django.contrib.auth.models import User
 from django.db.models import Max
 from django.http import Http404
 from django.template import Template, Context
@@ -227,6 +226,8 @@ class CaseTagFormTest(test.TestCase):
 class TestOperateComponentView(BasePlanCase):
     """Tests for operating components on cases"""
 
+    auto_login = True
+
     @classmethod
     def setUpTestData(cls):
         super().setUpTestData()
@@ -262,8 +263,6 @@ class TestOperateComponentView(BasePlanCase):
                               'testcases.delete_testcasecomponent')
 
     def test_show_components_form(self):
-        self.client.login(username=self.tester.username, password='password')
-
         response = self.client.post(reverse('cases-get-component-form'),
                                     {'product': self.product.pk})
 
@@ -287,8 +286,6 @@ class TestOperateComponentView(BasePlanCase):
             html=True)
 
     def test_add_components(self):
-        self.client.login(username=self.tester.username, password='password')
-
         post_data = {
             'product': self.product.pk,
             'o_component': [self.comp_application.pk, self.comp_database.pk],
@@ -309,8 +306,6 @@ class TestOperateComponentView(BasePlanCase):
             self.assertTrue(case_components.exists())
 
     def test_remove_components(self):
-        self.client.login(username=self.tester.username, password='password')
-
         user_should_have_perm(self.tester,
                               'testcases.delete_testcasecomponent')
 
@@ -334,7 +329,6 @@ class TestOperateComponentView(BasePlanCase):
 
     def test_fail_to_remove_if_component_not_exist(self):
         user_should_have_perm(self.tester, 'testcases.delete_testcasecomponent')
-        self.login_tester()
 
         result = Component.objects.aggregate(max_pk=Max('pk'))
         nonexistent_id = result['max_pk'] + 1
@@ -353,7 +347,6 @@ class TestOperateComponentView(BasePlanCase):
         remove_component.side_effect = Exception
 
         user_should_have_perm(self.tester, 'testcases.delete_testcasecomponent')
-        self.login_tester()
 
         resp = self.client.post(reverse('cases-remove-component'), {
             'o_component': [self.comp_cli.pk, self.comp_api.pk],
@@ -373,6 +366,8 @@ class TestOperateComponentView(BasePlanCase):
 class TestOperateCategoryView(BasePlanCase):
     """Tests for operating category on cases"""
 
+    auto_login = True
+
     @classmethod
     def setUpTestData(cls):
         super().setUpTestData()
@@ -389,8 +384,6 @@ class TestOperateCategoryView(BasePlanCase):
         cls.case_category_url = reverse('cases-category')
 
     def test_show_categories_form(self):
-        self.client.login(username=self.tester.username, password='password')
-
         response = self.client.post(self.case_category_url,
                                     {'product': self.product.pk})
 
@@ -413,8 +406,6 @@ class TestOperateCategoryView(BasePlanCase):
             html=True)
 
     def test_update_cases_category(self):
-        self.client.login(username=self.tester.username, password='password')
-
         post_data = {
             'from_plan': self.plan.pk,
             'product': self.product.pk,
@@ -434,6 +425,8 @@ class TestOperateCategoryView(BasePlanCase):
 class TestOperateCasePlans(BasePlanCase):
     """Test operation in case' plans tab"""
 
+    auto_login = True
+
     @classmethod
     def setUpTestData(cls):
         super().setUpTestData()
@@ -461,18 +454,10 @@ class TestOperateCasePlans(BasePlanCase):
         cls.case_1.add_to_plan(cls.plan_test_case_plans)
         cls.case_1.add_to_plan(cls.plan_test_remove)
 
-        cls.plan_tester = User.objects.create_user(
-            username='plantester',
-            email='plantester@example.com',
-            password='password')
-
         cls.perm_add = 'testcases.add_testcaseplan'
         cls.perm_change = 'testcases.change_testcaseplan'
-        user_should_have_perm(cls.plan_tester, cls.perm_add)
-        user_should_have_perm(cls.plan_tester, cls.perm_change)
-
-    def setUp(self):
-        self.login_tester(self.plan_tester, 'password')
+        user_should_have_perm(cls.tester, cls.perm_add)
+        user_should_have_perm(cls.tester, cls.perm_change)
 
     def _contains_html(self, content, expected):
         from django.test.testcases import assert_and_parse_html
@@ -501,7 +486,7 @@ class TestOperateCasePlans(BasePlanCase):
             )
 
     def test_missing_permission_to_add(self):
-        remove_perm_from_user(self.plan_tester, self.perm_add)
+        remove_perm_from_user(self.tester, self.perm_add)
         response = self.client.post(
             reverse('case-add-to-plans', args=[self.case_1.pk]),
             {'plan': self.plan_test_add.pk}
@@ -509,7 +494,7 @@ class TestOperateCasePlans(BasePlanCase):
         self.assert403(response)
 
     def test_missing_permission_to_remove(self):
-        remove_perm_from_user(self.plan_tester, self.perm_change)
+        remove_perm_from_user(self.tester, self.perm_change)
         response = self.client.post(
             reverse('case-remove-from-plans', args=[self.case_1.pk]),
             {'plan': self.plan_test_remove.pk}
@@ -674,6 +659,8 @@ class TestOperateCaseTag(BasePlanCase):
 class TestEditCase(BasePlanCase):
     """Test edit view method"""
 
+    auto_login = True
+
     @classmethod
     def setUpTestData(cls):
         super().setUpTestData()
@@ -714,24 +701,19 @@ class TestEditCase(BasePlanCase):
         }
 
     def test_404_if_case_id_not_exist(self):
-        self.login_tester()
         url = reverse('case-edit', args=[99999])
         response = self.client.get(url)
         self.assert404(response)
 
     def test_404_if_from_plan_not_exist(self):
-        self.login_tester()
         response = self.client.get(self.case_edit_url, {'from_plan': 9999})
         self.assert404(response)
 
     def test_show_edit_page(self):
-        self.login_tester()
         response = self.client.get(self.case_edit_url)
         self.assert200(response)
 
     def test_edit_a_case(self):
-        self.login_tester()
-
         edit_data = self.edit_data.copy()
         new_summary = f'Edited: {self.case_1.summary}'
         edit_data['summary'] = new_summary
@@ -748,8 +730,6 @@ class TestEditCase(BasePlanCase):
         self.assertEqual(new_summary, edited_case.summary)
 
     def test_continue_edit_this_case_after_save(self):
-        self.login_tester()
-
         edit_data = self.edit_data.copy()
         edit_data['_continue'] = 'continue edit'
 
@@ -762,8 +742,6 @@ class TestEditCase(BasePlanCase):
         self.assertRedirects(response, redirect_url)
 
     def test_continue_edit_next_confirmed_case_after_save(self):
-        self.login_tester()
-
         edit_data = self.edit_data.copy()
         edit_data['_continuenext'] = 'continue edit next case'
 
@@ -776,8 +754,6 @@ class TestEditCase(BasePlanCase):
         self.assertRedirects(response, redirect_url)
 
     def test_continue_edit_next_non_confirmed_case_after_save(self):
-        self.login_tester()
-
         edit_data = self.edit_data.copy()
         edit_data['case_status'] = self.case_status_proposed.pk
         edit_data['_continuenext'] = 'continue edit next case'
@@ -791,8 +767,6 @@ class TestEditCase(BasePlanCase):
         self.assertRedirects(response, redirect_url)
 
     def test_return_to_plan_confirmed_cases_tab(self):
-        self.login_tester()
-
         edit_data = self.edit_data.copy()
         edit_data['_returntoplan'] = 'return to plan'
 
@@ -804,8 +778,6 @@ class TestEditCase(BasePlanCase):
         self.assertRedirects(response, redirect_url, target_status_code=301)
 
     def test_return_to_plan_review_cases_tab(self):
-        self.login_tester()
-
         edit_data = self.edit_data.copy()
         edit_data['case_status'] = self.case_status_proposed.pk
         edit_data['_returntoplan'] = 'return to plan'
@@ -821,6 +793,8 @@ class TestEditCase(BasePlanCase):
 class TestChangeCasesAutomated(BasePlanCase):
     """Test automated view method"""
 
+    auto_login = True
+
     @classmethod
     def setUpTestData(cls):
         super().setUpTestData()
@@ -838,8 +812,6 @@ class TestChangeCasesAutomated(BasePlanCase):
         cls.change_url = reverse('cases-automated')
 
     def test_update_automated(self):
-        self.login_tester()
-
         change_data = self.change_data.copy()
         change_data['o_is_automated'] = 'on'
 
@@ -851,8 +823,6 @@ class TestChangeCasesAutomated(BasePlanCase):
             self.assertEqual(1, case.is_automated)
 
     def test_update_manual(self):
-        self.login_tester()
-
         change_data = self.change_data.copy()
         change_data['o_is_manual'] = 'on'
 
@@ -865,8 +835,6 @@ class TestChangeCasesAutomated(BasePlanCase):
             self.assertEqual(0, case.is_automated)
 
     def test_update_automated_proposed(self):
-        self.login_tester()
-
         change_data = self.change_data.copy()
         change_data['o_is_automated_proposed'] = 'on'
 
@@ -878,8 +846,6 @@ class TestChangeCasesAutomated(BasePlanCase):
             self.assertTrue(case.is_automated_proposed)
 
     def test_fail_due_to_invalid_input(self):
-        self.login_tester()
-
         change_data = self.change_data.copy()
         result = TestCase.objects.aggregate(max_pk=Max('pk'))
         change_data['case'] = [result['max_pk'] + 1]
@@ -1096,6 +1062,8 @@ class TestPrintablePage(BasePlanCase):
 class TestCloneCase(BasePlanCase):
     """Test clone view method"""
 
+    auto_login = True
+
     @classmethod
     def setUpTestData(cls):
         super().setUpTestData()
@@ -1157,16 +1125,12 @@ class TestCloneCase(BasePlanCase):
         cls.clone_url = reverse('cases-clone')
 
     def test_refuse_if_missing_argument(self):
-        self.login_tester()
-
         # Refuse to clone cases if missing selectAll and case arguments
         response = self.client.get(self.clone_url, {})
 
         self.assertContains(response, 'At least one case is required')
 
     def test_show_clone_page_with_from_plan(self):
-        self.login_tester()
-
         response = self.client.get(self.clone_url, {
             'from_plan': self.plan.pk,
             'case': [self.case_1.pk, self.case_2.pk]
@@ -1194,8 +1158,6 @@ class TestCloneCase(BasePlanCase):
         self.assertEqual([self.case_1.pk, self.case_2.pk], case_ids)
 
     def test_show_clone_page_without_from_plan(self):
-        self.login_tester()
-
         response = self.client.get(self.clone_url, {'case': self.case_1.pk})
 
         self.assertNotContains(
@@ -1294,8 +1256,8 @@ class TestCloneCase(BasePlanCase):
 
             # Assert attachments
             if copy_attachment:
-                orig_attachments = orig_case.attachment.order_by('pk')
-                cloned_attachments = cloned_case.attachment.order_by('pk')
+                orig_attachments = orig_case.attachments.order_by('pk')
+                cloned_attachments = cloned_case.attachments.order_by('pk')
                 self.assertListEqual(list(orig_attachments),
                                      list(cloned_attachments))
 
@@ -1331,8 +1293,6 @@ class TestCloneCase(BasePlanCase):
                      keep_original_default_tester=False,
                      copy_component=False,
                      copy_attachment=False):
-        self.login_tester()
-
         post_data = {
             'plan': [item.pk for item in dest_plans],
             'case': [item.pk for item in orig_cases],
@@ -1596,6 +1556,8 @@ class TestSearchCases(BasePlanCase):
 class TestAddComponent(BasePlanCase):
     """Test AddComponentView"""
 
+    auto_login = True
+
     @classmethod
     def setUpTestData(cls):
         super().setUpTestData()
@@ -1608,11 +1570,10 @@ class TestAddComponent(BasePlanCase):
         cls.case_2.add_component(cls.component_cli)
 
     def setUp(self):
+        super().setUp()
         user_should_have_perm(self.tester, 'testcases.add_testcasecomponent')
         self.add_component_url = reverse('cases-add-component')
 
-        self.login_tester()
-
     def test_add_one_component(self):
         resp = self.client.post(self.add_component_url, {
             'product': self.product.pk,
@@ -1709,6 +1670,8 @@ class TestAddComponent(BasePlanCase):
 class TestIssueManagement(BaseCaseRun):
     """Test add and remove issue to and from a test case"""
 
+    auto_login = True
+
     @classmethod
     def setUpTestData(cls):
         super().setUpTestData()
@@ -1732,14 +1695,19 @@ class TestIssueManagement(BaseCaseRun):
             validate_regex=r'^\d+$',
         )
 
+        cls.disabled_tracker = f.IssueTrackerFactory(
+            service_url='http://bz1.localhost/',
+            issue_report_endpoint='/enter_bug.cgi',
+            tracker_product=cls.tracker_product,
+            validate_regex=r'^\d+$',
+            enabled=False
+        )
+
         # Used for testing removing issue from test case.
         cls.case_2_delete_issue_url = reverse('cases-delete-issue', args=[cls.case_2.pk])
         cls.case_2.add_issue('67890', cls.issue_tracker)
         cls.case_2.add_issue('78901', cls.issue_tracker)
 
-    def setUp(self):
-        self.login_tester()
-
     def tearDown(self):
         self.client.logout()
         remove_perm_from_user(self.tester, self.perm_add_issue)
@@ -1914,10 +1882,28 @@ class TestIssueManagement(BaseCaseRun):
         self.assert400(resp)
         self.assertIn(f'Test case {case_id} does not exist.', resp.json()['message'])
 
+    def test_bad_request_if_an_issue_tracker_is_not_enabled(self):
+        user_should_have_perm(self.tester, self.perm_add_issue)
+
+        url = reverse('cases-add-issue', args=[self.case_1.pk])
+
+        resp = self.client.post(url, data={
+            'issue_key': self.fake_issue_key,
+            'tracker': self.disabled_tracker.pk,
+        })
+
+        self.assert400(resp)
+        self.assertIn(
+            f'Issue tracker "{self.disabled_tracker.name}" is not enabled',
+            resp.json()['message']
+        )
+
 
 class TestNewCase(BasePlanCase):
     """Test create a new case"""
 
+    auto_login = True
+
     @classmethod
     def setUpTestData(cls):
         super(TestNewCase, cls).setUpTestData()
@@ -1930,8 +1916,8 @@ class TestNewCase(BasePlanCase):
         user_should_have_perm(cls.tester, 'testcases.add_testcase')
 
     def setUp(self):
+        super().setUp()
         self.new_case_url = reverse('cases-new')
-        self.login_tester()
 
     def test_open_page(self):
         """Test open the page to create a new case"""
@@ -2382,7 +2368,7 @@ class TestCaseSimpleCaseRunView(BaseCaseRun):
             f'<p>{self.case_run_2.notes}</p>',
             '<li class="grey" style="border:none;margin:0px;padding:0px">'
             'No comments found.</li>',
-            '<li class="grey">No log found.</li>'
+            '<td colspan="5" class="empty-message-row">No logs.</td>'
         ]
         for item in expected_content:
             self.assertContains(resp, item, html=True)
@@ -2506,11 +2492,10 @@ class TestCaseCaseRunDetailPanelView(BaseCaseRun):
             '<ul class="ul-no-format"><li class="grey">No component found</li></ul>',
             '<ul class="ul-no-format"><li class="grey">No tag found</li></ul>',
 
-            f'<h4 class="borderB">Test Log <span>'
-            f'[<a href="javascript:void(0);" class="js-add-testlog" '
-            f'data-params="[{case_run.case_id}, {case_run.pk}]">Add</a>]'
-            f'</span></h4>'
-            f'<div class="content"><ul class="ul-format"></ul></div>',
+            '<h4 class="borderB">Test Log <span>'
+            '[<a href="javascript:void(0);" class="js-add-testlog">Add</a>]'
+            '</span></h4>'
+            '<div class="content"><ul class="ul-format"></ul></div>',
 
             '<h4>Actions</h4><div class="content"></div>',
             '<h4>Breakdown</h4><div class="content"></div>',
